@@ -2,8 +2,8 @@ from pequod_cli.service import CoreService
 
 
 class ClusterController(CoreService):
-    def __init__(self, core_seeds, credentials):
-        super().__init__(core_seeds, 'cluster-controller', credentials)
+    def __init__(self, endpoints, credentials):
+        super().__init__(endpoints, 'cluster-controller', credentials)
 
     def get_state(self):
         return self.get('/state')
\ No newline at end of file
