@@ -2,10 +2,13 @@
 from operator import itemgetter
 
 from io import BytesIO
+
 from netaddr.ip import IPNetwork
 from pathlib import Path
+import pequod_cli
 from pequod_cli.console import action, ok, error, print_table
 from pequod_cli.context import Context
+from pequod_cli.service import CORE_PORTS
 from pequod_cli.utils import generate_random_name, AliasedGroup
 import click
 import collections
@@ -21,11 +24,17 @@ DEFAULT_CONFIG_FILE='~/.pequod-cli.yaml'
 
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 
+def print_version(ctx, param, value):
+    if not value or ctx.resilient_parsing:
+        return
+    click.echo('Pequod CLI {}'.format(pequod_cli.__version__))
+    ctx.exit()
 
 @click.group(invoke_without_command=True, cls=AliasedGroup, context_settings=CONTEXT_SETTINGS)
 @click.option('--config-file', help='Use alternative config file', default=DEFAULT_CONFIG_FILE, metavar='PATH')
 @click.option('-c', '--cluster', envvar='PEQUOD_CLUSTER', help='Name of the cluster to use', default='default', metavar='NAME')
 @click.option('-v', '--verbose', help='Verbose logging', is_flag=True)
+@click.option('-V', '--version', is_flag=True, callback=print_version,  expose_value=False, is_eager=True)
 @click.pass_context
 def cli(ctx, config_file, cluster, verbose):
     """
@@ -41,30 +50,42 @@ def cli(ctx, config_file, cluster, verbose):
     if ctx.invoked_subcommand != 'login':
         if cluster not in data:
             raise click.UsageError('Cluster "{}" is not configured yet. Please use the "login" command or edit the configuration file {} manually.'.format(cluster, fn))
-        core_seeds = data[cluster]['core_seeds']
+        endpoints = data[cluster]['endpoints']
         credentials = data[cluster]['credentials']
     else:
-        core_seeds = []
+        endpoints = {}
         credentials = None
-    ctx.obj = Context(fn, cluster, core_seeds, credentials)
+    ctx.obj = Context(fn, cluster, endpoints, credentials)
     if not ctx.invoked_subcommand:
         _state(ctx.obj)
 
 
-class IPs(click.ParamType):
-    name = 'IPs'
+class Endpoints(click.ParamType):
+    name = 'Endpoints'
 
     def convert(self, value, param, ctx):
-        if isinstance(value, list):
+        if isinstance(value, dict):
             return value
-        return [ s.strip() for s in value.split(',')]
+        if value.startswith('{'):
+            return yaml.safe_load(value)
+        endpoints = {}
+        parts = [v.strip() for v in value.split(',')]
+
+        if parts[0].startswith('http'):
+            for k, port in CORE_PORTS.items():
+                endpoints[k] = [h.format(service=k) for h in parts]
+        else:
+            hosts = parts
+            for k, port in CORE_PORTS.items():
+                    endpoints[k] = ['http://{}:{}/'.format(h, port) for h in hosts]
+        return endpoints
 
 @cli.command()
-@click.option('-s', '--core-seeds', help='Core seed IPs', prompt='Core Seed IPs (comma separated)', metavar='IP, ..', type=IPs())
+@click.option('-e', '--endpoints', help='Endpoints', prompt='Service endpoints (list of IPs/URLs or complete JSON dict)', metavar='CONFIG', type=Endpoints())
 @click.option('-u', '--username', help='Username', prompt='Username', metavar='USER')
 @click.option('-p', '--password', help='Password', prompt=True, hide_input=True, metavar='PASSWD')
 @click.pass_context
-def login(click_context, core_seeds, username, password):
+def login(click_context, endpoints, username, password):
     """
     Login to a cluster and store credentials
     """
@@ -72,7 +93,7 @@ def login(click_context, core_seeds, username, password):
 
     credentials = {'username': username, 'password': password}
     ctx = click_context.obj
-    ctx = Context(ctx.config_file, ctx.cluster, core_seeds, credentials)
+    ctx = Context(ctx.config_file, ctx.cluster, endpoints, credentials)
 
     has_errors = False
     for svc in ctx.all_core_services:
@@ -97,7 +118,7 @@ def login(click_context, core_seeds, username, password):
     click.confirm('Save credentials for cluster {cluster} at {fn} in plaintext?'.format(cluster=ctx.cluster, fn=ctx.config_file), abort=True)
 
     action('Storing configuration for cluster {cluster} at {fn}..', cluster=ctx.cluster, fn=ctx.config_file)
-    data[ctx.cluster] = {'core_seeds': core_seeds, 'credentials': credentials}
+    data[ctx.cluster] = {'endpoints': endpoints, 'credentials': credentials}
 
     with open(ctx.config_file, 'w') as fd:
         yaml.safe_dump(data, fd)
@@ -329,6 +350,8 @@ def services(ctx):
     rows = []
     for row in data:
         manifest = ctx.registry.get_manifest(row['repository_name'], row['application_name'], row['application_version'])
+        if not manifest:
+            continue
         for service in manifest['services'].get('provide', []):
 
             rows.append({'service_name': service['service_name'],
@@ -442,7 +465,7 @@ def upload(ctx, files):
             raise click.UsageError('Template {}:{} {} already exists'.format(*identity))
         action('Uploading template {} version {} {}..'.format(*identity))
         with path.open('rb') as fd:
-            ctx.manager.put('/templates/{}/{}/{}'.format(template_name, template_version, image_type), data=fd.read(), headers={'Content-Type': 'application/octet-stream'}, timeout=300)
+            ctx.manager.put('/templates/{}/{}/{}'.format(template_name, template_version, image_type), data=fd, headers={'Content-Type': 'application/octet-stream'}, timeout=300)
         ok()
 
 
