@@ -2,8 +2,8 @@ from pequod_cli.service import CoreService
 
 
 class CoreManager(CoreService):
-    def __init__(self, core_seeds, credentials):
-        super().__init__(core_seeds, 'core-manager', credentials)
+    def __init__(self, endpoints, credentials):
+        super().__init__(endpoints, 'core-manager', credentials)
 
     def get_zones(self):
         data = self.get('/zones')
