@@ -12,6 +12,7 @@ CASSANDRA_PORT = 9042
 
 CORE_PORTS = {
     'auth-service': AUTH_PORT,
+    'file-replicator': REPLICATOR_PORT,
     'core-manager': MANAGER_PORT,
     'application-registry': REGISTRY_PORT,
     'cluster-controller': CONTROLLER_PORT,
@@ -67,8 +68,8 @@ class ClusterService:
         # (e.g. for template upload in Core Manager)
         endpoints.sort()
         last_error = None
-        for ip, port in endpoints:
-            url = 'http://{ip}:{port}{path}'.format(ip=format_ip(ip), port=port, path=path)
+        for ep in endpoints:
+            url = '{}{path}'.format(ep.rstrip('/'), path=path)
             try:
                 response = method(url, *args, **kwargs)
             except requests.ConnectionError as e:
@@ -144,21 +145,21 @@ class ClusterService:
 
 
 class CoreService(ClusterService):
-    def __init__(self, core_ips, service_name, credentials):
+    def __init__(self, endpoints, service_name, credentials):
         super().__init__(credentials)
         self.log = getLogger('pequod.core-service')
-        if isinstance(core_ips, str):
-            core_ips = core_ips.split(',')
-        self.core_ips = set(core_ips)
-        self.master_ip = None
+        self._endpoints = set(endpoints.get(service_name, []))
+        self._rotated_endpoints = sorted(self._endpoints)
+        self.master_endpoint = None
+        self.endpoints
         self.service_name = service_name
         self.port = CORE_PORTS[service_name]
 
     @property
     def endpoints(self):
-        if self.master_ip:
-            return [(self.master_ip, self.port)]
-        return [(ip, self.port) for ip in self.core_ips]
+        if self.master_endpoint:
+            return [self.master_endpoint]
+        return list(self._endpoints)
 
     @property
     def name(self):
@@ -173,11 +174,13 @@ class CoreService(ClusterService):
                 master = e.response.headers.get('X-Pequod-Master')
                 if not master:
                     raise
-                self.master_ip = parse_header_ip(master)
                 url = urllib.parse.urlparse(e.response.url)
-                for ip in self.core_ips:
-                    if ip != self.master_ip and ip != url.hostname.strip('[]'):
-                        self.master_ip = ip
+                for ep in self._rotated_endpoints:
+                    ep_url = urllib.parse.urlparse(ep)
+                    if ep_url.hostname != url.hostname:
+                        self.master_endpoint = ep
+                        break
+                self._rotated_endpoints = self._rotated_endpoints[1:] + self._rotated_endpoints[:1]
 
                 return super()._request(method_name, method, path, use_credentials, *args, **kwargs)
             raise
